<?php

namespace MyFastApp;

use MyFastApp\ErrorMessage;

require_once ABSPATH . "/wp-admin/includes/misc.php";
require_once ABSPATH . "/wp-admin/includes/file.php";
require_once ABSPATH . "/wp-admin/includes/class-wp-upgrader-skin.php";
require_once ABSPATH . "/wp-admin/includes/class-wp-upgrader.php";
require_once ABSPATH . "/wp-admin/includes/class-theme-upgrader.php";
require_once ABSPATH . "/wp-includes/class-wp-error.php";

require_once "Helper.php";
require_once "class-controller.php";
require_once "class-error.php";

class REST_API_Controller
{

    private $RESTroutes = [

        // [route, methods, callback, permission_callback]

        ["/userauth", "POST", "authenticate_user", "apikey_check"],

        ["/apikey", "GET", "get_apikey", "wordpress_check"],

        ["/settings", "GET", "get_settings", "wpApikey_check"],
        ["/settings", "POST", "update_settings", "wpApikey_check"],

        ["/menuItems", "GET", "get_menu_items", "wpApikey_check"],
        ["/menuItems", "POST", "update_menu_items", "wpApikey_check"],

        ["/resetSettings", "DELETE", "reset_settings", "wpApikey_check"],

        ["/checkPendingConfigChanges", "GET", "check_pending_config_changes", "wpApikey_check"],

        ["/live-config.json", "GET", "get_live_configuration", "apikey_check"],
        ["/updateLiveConfig", "POST", "update_live_configuration", "wpApikey_check"],

        ["/app-config.json", "GET", "get_app_configuration", "apikey_check"],
        ["/updateAppConfig", "POST", "update_app_configuration", "wpApikey_check"],

    ];

    /* --- Common Methods ------------------ */

    public function __construct()
    {
        $this->namespace = "myfastapp/v1";
    }

    public function register_routes()
    {
        foreach ($this->RESTroutes as $route) {
            $this->register_REST_route($route);
        }
    }

    private function register_REST_route($route)
    {
        register_rest_route(
            $this->namespace,
            $route[0],
            array(
                array(
                    "methods" => $route[1],
                    "callback" => array($this, $route[2]),
                    "permission_callback" => array($this, $route[3]),
                )
            )
        );
    }

    public function wordpress_check($request)
    {
        $wpNonce = wp_create_nonce("wp_rest");
        $requestNonce = $request->get_header("X-WP-Nonce");
        if ($wpNonce !== $requestNonce) {
            return false;
        }

        return true;
    }

    public function wpApikey_check($request)
    {
        $storedApiKey = unserialize(get_option("plugin_myfastapp_apitoken"));
        $requestApiKey = $request->get_header("AppBuilderApiKey");
        if (
            !$this->wordpress_check($request)
            || $storedApiKey !== $requestApiKey
        ) {
            return false;
        }

        return true;
    }

    public function apikey_check($request)
    {
        $storedApiKey = unserialize(get_option("plugin_myfastapp_apitoken"));
        $requestApiKey = $request->get_header("AppBuilderApiKey");
        if ($storedApiKey !== $requestApiKey) {
            return false;
        }

        return true;
    }

    public function no_check($request)
    {
        return true;
    }

    /* --- Security Methods ------------------ */

    public function authenticate_user($request)
    {
        try {

            $credentials = json_decode($request->get_body(), true);

            $encryptedUsername = $credentials["username"];
            $encryptedPassword = $credentials["password"];

            $salt = $this->get_today_salt();
            $username = $this->decrypt_text($salt, $encryptedUsername);
            $password = $this->decrypt_text($salt, $encryptedPassword);

            $user = wp_authenticate($username, $password);

            if ($user == null) {
                return new \WP_Error('user_unknown', 'User is unknown or invalid', array('status' => 401));
            }
            if (is_wp_error($user)) {
                return new \WP_Error('wrong_credentials', 'User is not authorized', array('status' => 401));
            }

            // {
            //     "data": {
            //         "ID": "3",
            //         "user_login": "",
            //         "user_pass": "",
            //         "user_nicename": "",
            //         "user_email": "",
            //         "user_url": "",
            //         "user_registered": "2022-02-10 20:43:32",
            //         "user_activation_key": "",
            //         "user_status": "",
            //         "display_name": ""
            //     },
            //     "ID": 3,
            //     "caps": {
            //         "contributor": true
            //     },
            //     "cap_key": "wp_capabilities",
            //     "roles": [
            //         "contributor"
            //     ],
            //     "allcaps": {
            //         "edit_posts": true,
            //         "read": true,
            //         "level_1": true,
            //         "level_0": true,
            //         "delete_posts": true,
            //         "contributor": true
            //     },
            //     "filter": null
            // }

            $response = [
                "UserId" => $user->ID,
                "Username" => $user->user_login,
                "UserEmail" => $user->user_email,
                "UserDisplayName" => $user->display_name,
                "UserRoles" => $user->roles,
            ];

            return rest_ensure_response($response);
        } catch (\Exception $ex) {
            write_log_error($ex->getMessage());
            return rest_ensure_response(new ErrorMessage(__("Unable to authenticate user", "my-fastapp")));
        }
    }

    public function get_apikey($request)
    {
        try {
            $apiKey = unserialize(get_option("plugin_myfastapp_apitoken"));
            return rest_ensure_response($apiKey);
        } catch (\Exception $ex) {
            write_log_error($ex->getMessage());
            return (new ErrorMessage(__("Unable to get API Key", "my-fastapp")));
        }
    }

    /* --- Settings Methods  ------------------ */

    public function get_settings($request)
    {
        try {
            $settings = json_decode(get_option("plugin_myfastapp_settings"));
            if (empty($settings))
                $settings = json_decode("{}");

            return rest_ensure_response($settings);
        } catch (\Exception $ex) {
            write_log_error($ex->getMessage());
            return (new ErrorMessage(__("Unable to obtain settings", "my-fastapp")));
        }
    }

    public function update_settings($request)
    {
        try {
            update_option("plugin_myfastapp_settings", $request->get_body());
        } catch (\Exception $ex) {
            write_log_error($ex->getMessage());
            return rest_ensure_response(new ErrorMessage(__("Unable to update settings", "my-fastapp")));
        }
    }

    public function get_menu_items($request)
    {
        try {
            $menuItems = json_decode(get_option("plugin_myfastapp_items"));
            if (empty($menuItems))
                $menuItems = json_decode("[]");

            return rest_ensure_response($menuItems);
        } catch (\Exception $ex) {
            write_log_error($ex->getMessage());
            return rest_ensure_response(new ErrorMessage(__("Unable to obtain menu items", "my-fastapp")));
        }
    }

    public function update_menu_items($request)
    {
        try {
            update_option("plugin_myfastapp_items", $request->get_body());
        } catch (\Exception $ex) {
            write_log_error($ex->getMessage());
            return rest_ensure_response(new ErrorMessage(__("Unable to update menu items", "my-fastapp")));
        }
    }

    public function reset_settings($request)
    {
        try {
            update_option("plugin_myfastapp_settings", NULL);
            update_option("plugin_myfastapp_items", NULL);
        } catch (\Exception $ex) {
            write_log_error($ex->getMessage());
            return rest_ensure_response(new ErrorMessage(__("Unable to update settings", "my-fastapp")));
        }
    }

    public function get_live_configuration($request)
    {
        try {
            $config = json_decode(get_option("plugin_myfastapp_live_config"));
            if (empty($config))
                $config = json_decode("{}");
            return rest_ensure_response($config);
        } catch (\Exception $ex) {
            write_log_error($ex->getMessage());
            return rest_ensure_response(new ErrorMessage(__("Unable to get live config", "my-fastapp")));
        }
    }

    public function update_live_configuration()
    {
        try {
            $settings = json_decode(get_option("plugin_myfastapp_settings"), true);
            $menuItems = json_decode(get_option("plugin_myfastapp_items"), true);
            $config  = $this->create_configuration($settings, $menuItems);
            update_option("plugin_myfastapp_live_config", json_encode($config));

            $externalApiUrl = get_option("plugin_myfastapp_external_api_uri");
            $apiKey = unserialize(get_option("plugin_myfastapp_apitoken"));
            // Do not use str_contains, it's only supported on PHP8+
            $isDev = strpos($externalApiUrl, "localhost") !== false;
            $response = wp_remote_post(
                $externalApiUrl . "CreateConfigCopy?configType=LiveConfig",
                [
                    "headers" => [
                        "Content-Type" => "application/json",
                        "AppBuilderApiKey" => $apiKey,
                    ],
                    "body" => json_encode($config),
                    "sslverify" => $isDev ? false : true,
                    "timeout" => 60
                ]
            );
            $storedFile = json_decode($response["body"], true);
            return rest_ensure_response($storedFile);
        } catch (\Exception $ex) {
            write_log_error($ex->getMessage());
            return rest_ensure_response(new ErrorMessage(__("Unable to update live config", "my-fastapp")));
        }
    }

    public function get_app_configuration($request)
    {
        try {
            $config = json_decode(get_option("plugin_myfastapp_app_config"));
            if (empty($config))
                $config = json_decode("{}");
            return rest_ensure_response($config);
        } catch (\Exception $ex) {
            write_log_error($ex->getMessage());
            return rest_ensure_response(new ErrorMessage(__("Unable to get app config", "my-fastapp")));
        }
    }

    public function update_app_configuration()
    {
        try {
            $settings = json_decode(get_option("plugin_myfastapp_settings"), true);
            $menuItems = json_decode(get_option("plugin_myfastapp_items"), true);
            $config  = $this->create_configuration($settings, $menuItems);
            update_option("plugin_myfastapp_app_config", json_encode($config));

            $externalApiUrl = get_option("plugin_myfastapp_external_api_uri");
            $apiKey = unserialize(get_option("plugin_myfastapp_apitoken"));
            $isDev = strpos($externalApiUrl, "localhost") !== false;
            $response = wp_remote_post(
                $externalApiUrl . "CreateConfigCopy?configType=AppConfig",
                [
                    "headers" => [
                        "Content-Type" => "application/json",
                        "AppBuilderApiKey" => $apiKey,
                    ],
                    "body" => json_encode($config),
                    "sslverify" => $isDev ? false : true,
                    "timeout" => 60
                ]
            );
            $storedFile = json_decode($response["body"], true);
            return rest_ensure_response($storedFile);
        } catch (\Exception $ex) {
            write_log_error($ex->getMessage());
            return rest_ensure_response(new ErrorMessage(__("Unable to update app config", "my-fastapp")));
        }
    }

    public function check_pending_config_changes()
    {
        try {

            $settings = json_decode(get_option("plugin_myfastapp_settings"), true);
            if (empty($settings))
                $settings = json_decode("{}");

            $menuItems = json_decode(get_option("plugin_myfastapp_items"), true);
            if (empty($menuItems))
                $menuItems = json_decode("[]");

            $currentConfig  = json_encode($this->create_configuration($settings, $menuItems));

            $liveConfig = get_option("plugin_myfastapp_live_config");
            $isLiveConfigUpdated = $currentConfig == $liveConfig;

            $appConfig = get_option("plugin_myfastapp_app_config");
            $isAppConfigUpdated = $currentConfig == $appConfig;

            $configStatus = [
                "isLiveConfigUpdated" => $isLiveConfigUpdated,
                "isAppConfigUpdated" => $isAppConfigUpdated
            ];
            return rest_ensure_response($configStatus);
        } catch (\Exception $ex) {
            write_log_error($ex->getMessage());
            return rest_ensure_response(new ErrorMessage(__("Unable to check pending config", "my-fastapp")));
        }
    }

    private function create_configuration($settings, $menuItems)
    {

        if ($settings == json_decode("{}", true) || $settings == json_decode("{}", false) || $menuItems == json_decode("[]", true) || $menuItems == json_decode("[]", false))
            return [];

        if (!array_key_exists("appTitle", $settings))
            return [];

        $result = [
            "ApiVersion" => "v1",
            "Data" => [
                "ShareAppAndroid" => $this->get_string_or_default("shareAppAndroid", $settings),
                "ShareAppIos" => $this->get_string_or_default("shareAppIos", $settings),
                "Environment" => [
                    "BundleId" => $this->get_string_or_default("bundleId", $settings),
                    "VersionName" => $this->get_string_or_default("versionNumber", $settings),
                    "iOSEnableDeploy" => $this->get_bool_or_default("enableiOSBuild", $settings),
                    "AndroidEnableDeploy" => $this->get_bool_or_default("enableAndroidBuild", $settings),
                    "AppPrimaryColor" => $this->get_color_or_default("appPrimaryColor", $settings),
                    "AppSecondaryColor" => $this->get_color_or_default("appSecondaryColor", $settings),
                    "EntryTextColor" => $this->get_color_or_default("entryTextColor", $settings),
                    "EntryBackgroundColor" => $this->get_color_or_default("entryBackgroundColor", $settings),
                    "ErrorTextColor" => $this->get_color_or_default("errorTextColor", $settings),
                    "AppTitleColor" => $this->get_color_or_default("appTitleColor", $settings),
                    "AppTitle" => $this->get_string_or_default("appTitle", $settings),
                    "AppMenuHeaderTextColor" => $this->get_color_or_default("colorHeaderMenu", $settings),
                    "AppMenuHeaderBackgroundColor" => $this->get_color_or_default("menuTitleBackgroundColor", $settings),
                    "AppMenuHeaderBackgroundImage" => $this->get_string_or_default("menuHeaderBackgroundImage", $settings),
                    "AppMenuPrimaryColor" => $this->get_color_or_default("menuItemsTextColor", $settings),
                    "AppMenuSecondaryColor" => $this->get_color_or_default("menuItemsBackgroundColor", $settings),
                    "AppMenuBackgroundColor" => $this->get_color_or_default("menuBackgroundColor", $settings),
                    "AppMenuTitle" => $this->get_string_or_default("menuTitle", $settings),
                    "AppMenuDisabled" => !$this->get_bool_or_default("enableAppMenu", $settings),
                    "AppMenuLayout" => $this->get_value_or_default("appMenuLayout", $settings, "Drawer"),
                    "HomeUrl" => $this->get_string_or_default("homeUrl", $settings),
                    "AppSplashScreenUrl" => $this->get_string_or_default("appSplashImage", $settings),
                    "OneSignalAppId" => $this->get_string_or_default("oneSignalAppId", $settings),
                    "SendGridApiKey" => $this->get_string_or_default("sendGripApiKey", $settings),
                    "UseLightStatusBar" => $this->get_bool_or_default("useLightStatusBar", $settings),
                    "UseUserLogin" => $this->get_bool_or_default("useUserLogin", $settings),
                    "UserLoginType" => $this->get_value_or_default("userLoginType", $settings, "Always"),
                    "LoginPageImage" => $this->get_string_or_default("loginPageImage", $settings),
                    "InternalApiUrl" => sprintf("%smyfastapp/v1/", esc_url_raw(rest_url())),
                    "ExternalApiUrl" => get_option("plugin_myfastapp_external_api_uri"),

                    // OBSOLETE
                    "AppMenuHeaderHeight" => 0,
                ],
                "Menus" => [],
            ],
        ];

        $parsedMenuitems = [];
        foreach ($menuItems as $menuItem) {
            if ($menuItem["isVisible"] == true) {
                $parsedMenuItem = [
                    "Title" => $this->get_string_or_default("title", $menuItem),
                    "BackgroundColor" => $this->get_color_or_default("backgroundColor", $menuItem),
                    "BackgroundImage" => $this->get_string_or_default("backgroundImage", $menuItem),
                    "TextColor" => $this->get_color_or_default("textColor", $menuItem),
                    "FontSize" => $this->get_string_or_default("fontSize", $menuItem),
                    "FontStyle" => $this->get_string_or_default("fontStyle", $menuItem),
                    "Thickness" => $this->get_value_or_default("type", $menuItem, "OnLineWebView") == "Separator" ? $menuItem["props"]["thickness"] : 0,
                    "IconName" => "\u" . $this->get_value_or_default("iconCode", $menuItem, "f000"),
                    "ActionType" => $this->get_value_or_default("type", $menuItem, "OnLineWebView"),
                    "TargetParam" => $this->get_menuItem_targetParam($menuItem),
                    "Props" => array_key_exists("props", $menuItem) ? json_encode($menuItem["props"]) : null,
                    "TargetPlatform" => $this->get_value_or_default("platform", $menuItem, "ios, android"),
                    "IsProtected" => $this->get_bool_or_default("isProtected", $menuItem, false),
                    "HideIcon" => $this->get_bool_or_default("hideIcon", $menuItem, false),
                    "HideTitle" => $this->get_bool_or_default("hideTitle", $menuItem, false),
                    // We are forcing this to support some features
                    //"SendCustomHeaders" => array_key_exists("sendCustomHeaders", $menuItem) ? $menuItem["sendCustomHeaders"] : false
                    "SendCustomHeaders" => true
                ];

                if (
                    $menuItem["type"] == "EmptyLine"
                    || $menuItem["type"] == "Separator"
                ) {

                    $parsedMenuItem["Title"] = null;
                    $parsedMenuItem["IconName"] = null;
                }

                $parsedMenuitems[] = $parsedMenuItem;
            }
        }

        $result["Data"]["Menus"] = $parsedMenuitems;

        return $result;
    }

    private function get_menuItem_targetParam($menuItem)
    {
        if ($menuItem["props"] == null)
            return null;

        switch ($menuItem["type"]) {
            case "HomePage":
                return $this->url_or_wp_url($menuItem);
            case "BusinessCard":
                return $menuItem["props"]["templateId"];
            case "MailTo":
                return $menuItem["props"]["email"];
            case "EmptyLine":
                return null;
            case "OpenURL":
                return $this->url_or_wp_url($menuItem);
            case "PhoneCall":
                return $menuItem["props"]["phoneNumber"];
            case "PhotoEditor":
                return null;
            case "PhotoSend":
                return $menuItem["props"]["email"];
            case "QR":
                return $menuItem["props"]["url"];
            case "Separator":
                return null;
            case "SendSms":
                return $menuItem["props"]["phoneNumber"];
            case "OnLineWebView":
                return $this->url_or_wp_url($menuItem);
        }
    }

    private function url_or_wp_url($menuItem)
    {
        return $menuItem["props"]["isAbsoluteUrl"] ?
            $menuItem["props"]["url"] : (array_key_exists("wpUrl", $menuItem["props"]) ? $menuItem["props"]["wpUrl"] : $menuItem["props"]["url"]);
    }

    private function get_string_or_default($settingsKey, $settings)
    {
        return array_key_exists($settingsKey, $settings) ? $settings[$settingsKey] : null;
    }

    private function get_number_or_default($settingsKey, $settings)
    {
        return array_key_exists($settingsKey, $settings) ? $settings[$settingsKey] : 0;
    }

    private function get_bool_or_default($settingsKey, $settings)
    {
        return array_key_exists($settingsKey, $settings) ? $settings[$settingsKey] : false;
    }

    private function get_color_or_default($settingsKey, $settings)
    {
        return array_key_exists($settingsKey, $settings) ? $this->hex_color_to_xamarin($settings[$settingsKey]) : "#FFFFFFFF";
    }

    private function get_value_or_default($settingsKey, $settings, $default)
    {
        return array_key_exists($settingsKey, $settings) ? $settings[$settingsKey] : $default;
    }

    private function hex_color_to_xamarin($color)
    {
        if (strlen($color) == 4)
            return $color;

        $rgb = substr($color, 1, 6);
        $alpha = substr($color, 7, 2);

        return sprintf("#%s%s", $alpha, $rgb);
    }

    private function get_today_salt()
    {
        try {
            $externalApiUrl = get_option("plugin_myfastapp_external_api_uri");
            $apiKey = unserialize(get_option("plugin_myfastapp_apitoken"));
            // Do not use str_contains, it's only supported on PHP8+
            $isDev = strpos($externalApiUrl, "localhost") !== false;
            $response = wp_remote_get(
                $externalApiUrl . "GetTodaySalt",
                [
                    "headers" => [
                        "AppBuilderApiKey" => $apiKey,
                    ],
                    "sslverify" => $isDev ? false : true,
                    "timeout" => 60
                ]
            );
            $salt = base64_decode($response["body"]);
        } catch (\Exception $ex) {
            write_log($ex->getMessage());
            return rest_ensure_response(new ErrorMessage(__("Unable to get today salt", "my-fastapp")));
        }
        return $salt;
    }

    private function decrypt_text($salt, $cipherText)
    {
        $key = $salt;
        $iv =  $salt;
        return preg_replace('/\x03/', '', openssl_decrypt(base64_decode($cipherText), 'AES-128-CBC', $key, OPENSSL_RAW_DATA | OPENSSL_ZERO_PADDING, $iv));
    }
}

if (!function_exists('write_log')) {
    function write_log($log)
    {
        if (is_array($log) || is_object($log)) {
            error_log(print_r($log, true));
        } else {
            error_log($log);
        }
    }

    function write_log_error($log)
    {
        write_log("MyFastError: " . $log);
    }
}
